@@ -1,12 +1,14 @@
 #!/usr/bin/env bash
 
-# This script handles any custom processing of the spec file generated using the `post-upstream-clone`
-# action and gets used by the fix-spec-file action in .packit.yaml.
+# This script handles any custom processing of the spec file using the `fix-spec-file`
+# action in .packit.yaml.
 
-set -eo pipefail
+set -eox pipefail
+
+PACKAGE=podman
 
 # Set path to rpm spec file
-SPEC_FILE=rpm/podman.spec
+SPEC_FILE=rpm/$PACKAGE.spec
 
 # Get Version from HEAD
 VERSION=$(grep '^const RawVersion' version/rawversion/version.go | cut -d\" -f2)
@@ -15,10 +17,7 @@ VERSION=$(grep '^const RawVersion' version/rawversion/version.go | cut -d\" -f2)
 RPM_VERSION=$(echo $VERSION | sed -e 's/-/~/')
 
 # Generate source tarball from HEAD
-git archive --prefix=podman-$VERSION/ -o podman-$VERSION.tar.gz HEAD
-
-# rpmbuild expects source tarball in the same dir as spec file
-mv podman-$VERSION.tar.gz rpm/
+git-archive-all -C $(git rev-parse --show-toplevel) --prefix=$PACKAGE-$VERSION/ rpm/$PACKAGE-$VERSION.tar.gz
 
 # RPM Spec modifications
 
@@ -26,12 +25,14 @@ mv podman-$VERSION.tar.gz rpm/
 sed -i "s/^Version:.*/Version: $RPM_VERSION/" $SPEC_FILE
 
 # Use Packit's supplied variable in the Release field in rpm spec.
-# podman.spec is generated using `rpkg spec --outdir ./` as mentioned in the
-# `post-upstream-clone` action in .packit.yaml.
 sed -i "s/^Release:.*/Release: $PACKIT_RPMSPEC_RELEASE%{?dist}/" $SPEC_FILE
 
+# Ensure last part of the release string is the git shortcommit without a
+# prepended "g"
+sed -i "/^Release: $PACKIT_RPMSPEC_RELEASE%{?dist}/ s/\(.*\)g/\1/" $SPEC_FILE
+
 # Use above generated tarball as Source in rpm spec
-sed -i "s/^Source0:.*.tar.gz/Source0: podman-$VERSION.tar.gz/" $SPEC_FILE
+sed -i "s/^Source0:.*.tar.gz/Source0: $PACKAGE-$VERSION.tar.gz/" $SPEC_FILE
 
 # Update setup macro to use the correct build dir
 sed -i "s/^%autosetup.*/%autosetup -Sgit -n %{name}-$VERSION/" $SPEC_FILE
