@@ -19,15 +19,17 @@ set -eo pipefail
 # shellcheck source=contrib/cirrus/lib.sh
 source $(dirname $0)/lib.sh
 
+showrun echo "starting"
+
 function _run_validate() {
     # TODO: aarch64 images need python3-devel installed
     # https://github.com/containers/automation_images/issues/159
-    bigto ooe.sh dnf install -y python3-devel
+    showrun bigto ooe.sh dnf install -y python3-devel
 
     # git-validation tool fails if $EPOCH_TEST_COMMIT is empty
     # shellcheck disable=SC2154
     if [[ -n "$EPOCH_TEST_COMMIT" ]]; then
-        make validate
+        showrun make validate
     else
         warn "Skipping git-validation since \$EPOCH_TEST_COMMIT is empty"
     fi
@@ -42,29 +44,29 @@ function _run_unit() {
         # shellcheck disable=SC2154
         die "$TEST_FLAVOR: Unsupported PODBIN_NAME='$PODBIN_NAME'"
     fi
-    make localunit
+    showrun make localunit
 }
 
 function _run_apiv2() {
     _bail_if_test_can_be_skipped test/apiv2
 
     (
-        make localapiv2-bash
+        showrun make localapiv2-bash
         source .venv/requests/bin/activate
-        make localapiv2-python
+        showrun make localapiv2-python
     ) |& logformatter
 }
 
 function _run_compose() {
     _bail_if_test_can_be_skipped test/compose
 
-    ./test/compose/test-compose |& logformatter
+    showrun ./test/compose/test-compose |& logformatter
 }
 
 function _run_compose_v2() {
     _bail_if_test_can_be_skipped test/compose
 
-    ./test/compose/test-compose |& logformatter
+    showrun ./test/compose/test-compose |& logformatter
 }
 
 function _run_int() {
@@ -80,20 +82,20 @@ function _run_sys() {
 }
 
 function _run_upgrade_test() {
-    _bail_if_test_can_be_skipped test/upgrade
+    _bail_if_test_can_be_skipped test/system test/upgrade
 
-    bats test/upgrade |& logformatter
+    showrun bats test/upgrade |& logformatter
 }
 
 function _run_bud() {
     _bail_if_test_can_be_skipped test/buildah-bud
 
-    ./test/buildah-bud/run-buildah-bud-tests |& logformatter
+    showrun ./test/buildah-bud/run-buildah-bud-tests |& logformatter
 }
 
 function _run_bindings() {
     # install ginkgo
-    make .install.ginkgo
+    showrun make .install.ginkgo
 
     # shellcheck disable=SC2155
     export PATH=$PATH:$GOSRC/hack:$GOSRC/test/tools/build
@@ -105,23 +107,23 @@ function _run_bindings() {
     fi
 
     (echo "$gitcommit_magic" && \
-        make testbindings) |& logformatter
+        showrun make testbindings) |& logformatter
 }
 
 function _run_docker-py() {
     source .venv/docker-py/bin/activate
-    make run-docker-py-tests
+    showrun make run-docker-py-tests
 }
 
 function _run_endpoint() {
-    make test-binaries
-    make endpoint
+    showrun make test-binaries
+    showrun make endpoint
 }
 
 function _run_minikube() {
     _bail_if_test_can_be_skipped test/minikube
     msg "Testing  minikube."
-    bats test/minikube |& logformatter
+    showrun bats test/minikube |& logformatter
 }
 
 exec_container() {
@@ -187,10 +189,10 @@ function _run_swagger() {
 
     # Swagger validation takes a significant amount of time
     msg "Pulling \$CTR_FQIN '$CTR_FQIN' (background process)"
-    bin/podman pull --quiet $CTR_FQIN &
+    showrun bin/podman pull --quiet $CTR_FQIN &
 
     cd $GOSRC
-    make swagger
+    showrun make swagger
 
     # Cirrus-CI Artifact instruction expects file here
     cp -v $GOSRC/pkg/api/swagger.yaml ./
@@ -209,7 +211,7 @@ eof
 
     msg "Waiting for backgrounded podman pull to complete..."
     wait %%
-    bin/podman run -it --rm --security-opt label=disable \
+    showrun bin/podman run -it --rm --security-opt label=disable \
         --env-file=$envvarsfile \
         -v $GOSRC:$GOSRC:ro \
         --workdir $GOSRC \
@@ -219,9 +221,9 @@ eof
 
 function _run_build() {
     # Ensure always start from clean-slate with all vendor modules downloaded
-    make clean
-    make vendor
-    make podman-release  # includes podman, podman-remote, and docs
+    showrun make clean
+    showrun make vendor
+    showrun make podman-release  # includes podman, podman-remote, and docs
 
     # Last-minute confirmation that we're testing the desired runtime.
     # This Can't Possibly Fail™ in regular CI; only when updating VMs.
@@ -252,7 +254,7 @@ function _run_altbuild() {
     cd $GOSRC
     case "$ALT_NAME" in
         *Each*)
-            git fetch origin
+            showrun git fetch origin
             # The make-and-check-size script, introduced 2022-03-22 in #13518,
             # runs 'make' (the original purpose of this check) against
             # each commit, then checks image sizes to make sure that
@@ -264,46 +266,62 @@ function _run_altbuild() {
             savedhead=$(git rev-parse HEAD)
             # Push to PR base. First run of the script will write size files
             pr_base=$(git merge-base origin/$DEST_BRANCH HEAD)
-            git checkout $pr_base
-            hack/make-and-check-size $context_dir
+            showrun git checkout $pr_base
+            showrun hack/make-and-check-size $context_dir
             # pop back to PR, and run incremental makes. Subsequent script
             # invocations will compare against original size.
-            git checkout $savedhead
-            git rebase $pr_base -x "hack/make-and-check-size $context_dir"
+            showrun git checkout $savedhead
+            showrun git rebase $pr_base -x "hack/make-and-check-size $context_dir"
             rm -rf $context_dir
             ;;
         *Windows*)
-            make podman-remote-release-windows_amd64.zip
-            make podman.msi
+            showrun make podman-remote-release-windows_amd64.zip
             ;;
         *RPM*)
-            make package
+            showrun make package
             ;;
-        FreeBSD*Cross)
-            make bin/podman.cross.freebsd.amd64
-            ;;
-        Alt*Cross)
+        Alt*x86*Cross)
             arches=(\
                 amd64
-                ppc64le
+                386)
+            _build_altbuild_archs "${arches[@]}"
+            ;;
+        Alt*ARM*Cross)
+            arches=(\
                 arm
-                arm64
-                386
-                s390x
+                arm64)
+            _build_altbuild_archs "${arches[@]}"
+            ;;
+        Alt*Other*Cross)
+            arches=(\
+                ppc64le
+                s390x)
+            _build_altbuild_archs "${arches[@]}"
+            ;;
+        Alt*MIPS*Cross)
+            arches=(\
                 mips
-                mipsle
+                mipsle)
+            _build_altbuild_archs "${arches[@]}"
+            ;;
+        Alt*MIPS64*Cross*)
+            arches=(\
                 mips64
                 mips64le)
-            for arch in "${arches[@]}"; do
-                msg "Building release archive for $arch"
-                make podman-release-${arch}.tar.gz GOARCH=$arch
-            done
+            _build_altbuild_archs "${arches[@]}"
             ;;
         *)
             die "Unknown/Unsupported \$$ALT_NAME '$ALT_NAME'"
     esac
 }
 
+function _build_altbuild_archs() {
+    for arch in "$@"; do
+        msg "Building release archive for $arch"
+        showrun make podman-release-${arch}.tar.gz GOARCH=$arch
+    done
+}
+
 function _run_release() {
     msg "podman info:"
     bin/podman info
@@ -342,6 +360,13 @@ function _run_gitlab() {
     return $ret
 }
 
+# TODO: enable fcos_image_build task in cirrus
+#function _run_fcos_image_build() {
+#    # FIXME: Doesn't seem to grab CTXDIR from .cirrus.yml
+#    CTXDIR="contrib/podman-next/fcos-podmanimage"
+#    podman build -t quay.io/podman/fcos:podman-next $CTXDIR
+#}
+
 # Name pattern for logformatter output file, derived from environment
 function output_name() {
     # .cirrus.yml defines this as a short readable string for web UI
@@ -395,17 +420,17 @@ dotest() {
         die "Found fallback podman '$fallback_podman' in \$PATH; tests require none, as a guarantee that we're testing the right binary."
     fi
 
-    make ${localremote}${testsuite} PODMAN_SERVER_LOG=$PODMAN_SERVER_LOG \
+    showrun make ${localremote}${testsuite} PODMAN_SERVER_LOG=$PODMAN_SERVER_LOG \
         |& logformatter
 }
 
 _run_machine() {
     # N/B: Can't use _bail_if_test_can_be_skipped here b/c content isn't under test/
-    make localmachine |& logformatter
+    showrun make localmachine |& logformatter
 }
 
 # Optimization: will exit if the only PR diffs are under docs/ or tests/
-# with the exception of any given arguments. E.g., don't run e2e or upgrade
+# with the exception of any given arguments. E.g., don't run e2e or unit
 # or bud tests if the only PR changes are in test/system.
 function _bail_if_test_can_be_skipped() {
     local head base diffs
@@ -520,4 +545,6 @@ if [ "$(type -t $handler)" != "function" ]; then
     die "Unknown/Unsupported \$TEST_FLAVOR=$TEST_FLAVOR"
 fi
 
-$handler
+showrun $handler
+
+showrun echo "finished"
