@@ -182,7 +182,8 @@ func (ic *ContainerEngine) ContainerRestart(ctx context.Context, namesOrIds []st
 	)
 	options := new(containers.RestartOptions)
 	if to := opts.Timeout; to != nil {
-		options.WithTimeout(int(*to))
+		timeout := util.ConvertTimeout(int(*to))
+		options.WithTimeout(int(timeout))
 	}
 	ctrs, rawInputs, err := getContainersAndInputByContext(ic.ClientCtx, opts.All, false, namesOrIds, opts.Filters)
 	if err != nil {
@@ -693,7 +694,12 @@ func (ic *ContainerEngine) ContainerStart(ctx context.Context, namesOrIds []stri
 		}
 	}
 	removeOptions := new(containers.RemoveOptions).WithVolumes(true).WithForce(false)
-	removeContainer := func(id string) {
+	removeContainer := func(id, CIDFile string) {
+		if CIDFile != "" {
+			if err := os.Remove(CIDFile); err != nil && !errors.Is(err, os.ErrNotExist) {
+				logrus.Warnf("Cleaning up CID file: %s", err)
+			}
+		}
 		reports, err := containers.Remove(ic.ClientCtx, id, removeOptions)
 		logIfRmError(id, err, reports)
 	}
@@ -722,7 +728,7 @@ func (ic *ContainerEngine) ContainerStart(ctx context.Context, namesOrIds []stri
 
 			if err != nil {
 				if ctr.AutoRemove {
-					removeContainer(ctr.ID)
+					removeContainer(ctr.ID, ctr.CIDFile)
 				}
 				report.ExitCode = define.ExitCode(report.Err)
 				report.Err = err
@@ -741,7 +747,7 @@ func (ic *ContainerEngine) ContainerStart(ctx context.Context, namesOrIds []stri
 					logrus.Errorf("Should restart: %v", shouldRestart)
 
 					if !shouldRestart && ctr.AutoRemove {
-						removeContainer(ctr.ID)
+						removeContainer(ctr.ID, ctr.CIDFile)
 					}
 				}()
 			}
@@ -827,7 +833,13 @@ func (ic *ContainerEngine) ContainerRun(ctx context.Context, opts entities.Conta
 	for _, w := range con.Warnings {
 		fmt.Fprintf(os.Stderr, "%s\n", w)
 	}
-	removeContainer := func(id string, force bool) error {
+	removeContainer := func(id, CIDFile string, force bool) error {
+		if CIDFile != "" {
+			if err := os.Remove(CIDFile); err != nil && !errors.Is(err, os.ErrNotExist) {
+				logrus.Warnf("Cleaning up CID file: %s", err)
+			}
+		}
+
 		removeOptions := new(containers.RemoveOptions).WithVolumes(true).WithForce(force)
 		reports, err := containers.Remove(ic.ClientCtx, id, removeOptions)
 		logIfRmError(id, err, reports)
@@ -837,7 +849,7 @@ func (ic *ContainerEngine) ContainerRun(ctx context.Context, opts entities.Conta
 	if opts.CIDFile != "" {
 		if err := util.CreateIDFile(opts.CIDFile, con.ID); err != nil {
 			// If you fail to create CIDFile then remove the container
-			_ = removeContainer(con.ID, true)
+			_ = removeContainer(con.ID, opts.CIDFile, true)
 			return nil, err
 		}
 	}
@@ -850,9 +862,7 @@ func (ic *ContainerEngine) ContainerRun(ctx context.Context, opts entities.Conta
 		if err != nil {
 			report.ExitCode = define.ExitCode(err)
 			if opts.Rm {
-				if rmErr := removeContainer(con.ID, true); rmErr != nil && !errors.Is(rmErr, define.ErrNoSuchCtr) {
-					logrus.Errorf("Container %s failed to be removed", con.ID)
-				}
+				_ = removeContainer(con.ID, opts.CIDFile, true)
 			}
 		}
 		return &report, err
@@ -873,7 +883,7 @@ func (ic *ContainerEngine) ContainerRun(ctx context.Context, opts entities.Conta
 
 		report.ExitCode = define.ExitCode(err)
 		if opts.Rm {
-			_ = removeContainer(con.ID, false)
+			_ = removeContainer(con.ID, opts.CIDFile, false)
 		}
 		return &report, err
 	}
@@ -889,7 +899,7 @@ func (ic *ContainerEngine) ContainerRun(ctx context.Context, opts entities.Conta
 			}
 
 			if !shouldRestart {
-				_ = removeContainer(con.ID, false)
+				_ = removeContainer(con.ID, opts.CIDFile, false)
 			}
 		}()
 	}
@@ -1049,7 +1059,7 @@ func (ic *ContainerEngine) ContainerStats(ctx context.Context, namesOrIds []stri
 	if options.Latest {
 		return nil, errors.New("latest is not supported for the remote client")
 	}
-	return containers.Stats(ic.ClientCtx, namesOrIds, new(containers.StatsOptions).WithStream(options.Stream).WithInterval(options.Interval))
+	return containers.Stats(ic.ClientCtx, namesOrIds, new(containers.StatsOptions).WithStream(options.Stream).WithInterval(options.Interval).WithAll(options.All))
 }
 
 // ShouldRestart reports back whether the container will restart.
