@@ -4,12 +4,15 @@
 package machine
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"net"
+	"net/http"
 	"net/url"
 	"os"
 	"path/filepath"
+	"strings"
 	"time"
 
 	"github.com/containers/storage/pkg/homedir"
@@ -17,22 +20,22 @@ import (
 )
 
 type InitOptions struct {
-	CPUS         uint64
-	DiskSize     uint64
-	IgnitionPath string
-	ImagePath    string
-	Volumes      []string
-	VolumeDriver string
-	IsDefault    bool
-	Memory       uint64
-	Name         string
-	TimeZone     string
-	URI          url.URL
-	Username     string
-	ReExec       bool
-	Rootful      bool
-	// The numerical userid of the user that called machine
-	UID string
+	CPUS               uint64
+	DiskSize           uint64
+	IgnitionPath       string
+	ImagePath          string
+	Volumes            []string
+	VolumeDriver       string
+	IsDefault          bool
+	Memory             uint64
+	Name               string
+	TimeZone           string
+	URI                url.URL
+	Username           string
+	ReExec             bool
+	Rootful            bool
+	UID                string // uid of the user that called machine
+	UserModeNetworking *bool  // nil = use backend/system default, false = disable, true = enable
 }
 
 type Status = string
@@ -45,72 +48,59 @@ const (
 	// Starting indicated the vm is in the process of starting
 	Starting           Status = "starting"
 	DefaultMachineName string = "podman-machine-default"
+	apiUpTimeout              = 20 * time.Second
 )
 
-type VirtProvider interface {
-	Artifact() Artifact
-	CheckExclusiveActiveVM() (bool, string, error)
-	Compression() ImageCompression
-	Format() ImageFormat
-	IsValidVMName(name string) (bool, error)
-	List(opts ListOptions) ([]*ListResponse, error)
-	LoadVMByName(name string) (VM, error)
-	NewMachine(opts InitOptions) (VM, error)
-	RemoveAndCleanMachines() error
-	VMType() VMType
-}
-
 type RemoteConnectionType string
 
 var (
 	SSHRemoteConnection     RemoteConnectionType = "ssh"
 	DefaultIgnitionUserName                      = "core"
-	ErrNoSuchVM                                  = errors.New("VM does not exist")
-	ErrVMAlreadyExists                           = errors.New("VM already exists")
-	ErrVMAlreadyRunning                          = errors.New("VM already running or starting")
-	ErrMultipleActiveVM                          = errors.New("only one VM can be active at a time")
-	ErrNotImplemented                            = errors.New("functionality not implemented")
 	ForwarderBinaryName                          = "gvproxy"
 )
 
 type Download struct {
 	Arch                  string
 	Artifact              Artifact
-	CompressionType       string
 	CacheDir              string
+	CompressionType       ImageCompression
+	DataDir               string
 	Format                ImageFormat
 	ImageName             string
 	LocalPath             string
 	LocalUncompressedFile string
 	Sha256sum             string
-	URL                   *url.URL
-	VMName                string
 	Size                  int64
+	URL                   *url.URL
+	VMKind                VMType
+	VMName                string
 }
 
 type ListOptions struct{}
 
 type ListResponse struct {
-	Name           string
-	CreatedAt      time.Time
-	LastUp         time.Time
-	Running        bool
-	Starting       bool
-	Stream         string
-	VMType         string
-	CPUs           uint64
-	Memory         uint64
-	DiskSize       uint64
-	Port           int
-	RemoteUsername string
-	IdentityPath   string
+	Name               string
+	CreatedAt          time.Time
+	LastUp             time.Time
+	Running            bool
+	Starting           bool
+	Stream             string
+	VMType             string
+	CPUs               uint64
+	Memory             uint64
+	DiskSize           uint64
+	Port               int
+	RemoteUsername     string
+	IdentityPath       string
+	UserModeNetworking bool
 }
 
 type SetOptions struct {
-	CPUs     *uint64
-	DiskSize *uint64
-	Memory   *uint64
-	Rootful  *bool
+	CPUs               *uint64
+	DiskSize           *uint64
+	Memory             *uint64
+	Rootful            *bool
+	UserModeNetworking *bool
 }
 
 type SSHOptions struct {
@@ -151,15 +141,17 @@ type DistributionDownload interface {
 	CleanCache() error
 }
 type InspectInfo struct {
-	ConfigPath     VMFile
-	ConnectionInfo ConnectionConfig
-	Created        time.Time
-	Image          ImageConfig
-	LastUp         time.Time
-	Name           string
-	Resources      ResourceConfig
-	SSHConfig      SSHConfig
-	State          Status
+	ConfigPath         VMFile
+	ConnectionInfo     ConnectionConfig
+	Created            time.Time
+	Image              ImageConfig
+	LastUp             time.Time
+	Name               string
+	Resources          ResourceConfig
+	SSHConfig          SSHConfig
+	State              Status
+	UserModeNetworking bool
+	Rootful            bool
 }
 
 func (rc RemoteConnectionType) MakeSSHURL(host, path, port, userName string) url.URL {
@@ -210,6 +202,17 @@ func GetDataDir(vmType VMType) (string, error) {
 	return dataDir, mkdirErr
 }
 
+// GetGLobalDataDir returns the root of all backends
+// for shared machine data.
+func GetGlobalDataDir() (string, error) {
+	dataDir, err := DataDirPrefix()
+	if err != nil {
+		return "", err
+	}
+
+	return dataDir, os.MkdirAll(dataDir, 0755)
+}
+
 // DataDirPrefix returns the path prefix for all machine data files
 func DataDirPrefix() (string, error) {
 	data, err := homedir.GetDataHome()
@@ -360,6 +363,8 @@ type HostUser struct {
 	Rootful bool
 	// UID is the numerical id of the user that called machine
 	UID int
+	// Whether one of these fields has changed and actions should be taken
+	Modified bool `json:"HostUserModified"`
 }
 
 // SSHConfig contains remote access information for SSH
@@ -400,3 +405,196 @@ func (v VMType) String() string {
 	}
 	return "qemu"
 }
+
+type APIForwardingState int
+
+const (
+	NoForwarding APIForwardingState = iota
+	ClaimUnsupported
+	NotInstalled
+	MachineLocal
+	DockerGlobal
+)
+
+func ParseVMType(input string, emptyFallback VMType) (VMType, error) {
+	switch strings.TrimSpace(strings.ToLower(input)) {
+	case "qemu":
+		return QemuVirt, nil
+	case "wsl":
+		return WSLVirt, nil
+	case "applehv":
+		return AppleHvVirt, nil
+	case "hyperv":
+		return HyperVVirt, nil
+	case "":
+		return emptyFallback, nil
+	default:
+		return QemuVirt, fmt.Errorf("unknown VMType `%s`", input)
+	}
+}
+
+type VirtProvider interface { //nolint:interfacebloat
+	Artifact() Artifact
+	CheckExclusiveActiveVM() (bool, string, error)
+	Compression() ImageCompression
+	Format() ImageFormat
+	IsValidVMName(name string) (bool, error)
+	List(opts ListOptions) ([]*ListResponse, error)
+	LoadVMByName(name string) (VM, error)
+	NewMachine(opts InitOptions) (VM, error)
+	NewDownload(vmName string) (Download, error)
+	RemoveAndCleanMachines() error
+	VMType() VMType
+}
+
+type Virtualization struct {
+	artifact    Artifact
+	compression ImageCompression
+	format      ImageFormat
+	vmKind      VMType
+}
+
+func (p *Virtualization) Artifact() Artifact {
+	return p.artifact
+}
+
+func (p *Virtualization) Compression() ImageCompression {
+	return p.compression
+}
+
+func (p *Virtualization) Format() ImageFormat {
+	return p.format
+}
+
+func (p *Virtualization) VMType() VMType {
+	return p.vmKind
+}
+
+func (p *Virtualization) NewDownload(vmName string) (Download, error) {
+	cacheDir, err := GetCacheDir(p.VMType())
+	if err != nil {
+		return Download{}, err
+	}
+
+	dataDir, err := GetDataDir(p.VMType())
+	if err != nil {
+		return Download{}, err
+	}
+
+	return Download{
+		Artifact:        p.Artifact(),
+		CacheDir:        cacheDir,
+		CompressionType: p.Compression(),
+		DataDir:         dataDir,
+		Format:          p.Format(),
+		VMKind:          p.VMType(),
+		VMName:          vmName,
+	}, nil
+}
+
+func NewVirtualization(artifact Artifact, compression ImageCompression, format ImageFormat, vmKind VMType) Virtualization {
+	return Virtualization{
+		artifact,
+		compression,
+		format,
+		vmKind,
+	}
+}
+
+func WaitAndPingAPI(sock string) {
+	client := http.Client{
+		Transport: &http.Transport{
+			DialContext: func(context.Context, string, string) (net.Conn, error) {
+				con, err := net.DialTimeout("unix", sock, apiUpTimeout)
+				if err != nil {
+					return nil, err
+				}
+				if err := con.SetDeadline(time.Now().Add(apiUpTimeout)); err != nil {
+					return nil, err
+				}
+				return con, nil
+			},
+		},
+	}
+
+	resp, err := client.Get("http://host/_ping")
+	if err == nil {
+		defer resp.Body.Close()
+	}
+	if err != nil || resp.StatusCode != 200 {
+		logrus.Warn("API socket failed ping test")
+	}
+}
+
+func (dl Download) NewFcosDownloader(imageStream FCOSStream) (DistributionDownload, error) {
+	info, err := dl.GetFCOSDownload(imageStream)
+	if err != nil {
+		return nil, err
+	}
+	urlSplit := strings.Split(info.Location, "/")
+	dl.ImageName = urlSplit[len(urlSplit)-1]
+	downloadURL, err := url.Parse(info.Location)
+	if err != nil {
+		return nil, err
+	}
+
+	// Complete the download struct
+	dl.Arch = GetFcosArch()
+	// This could be eliminated as a struct and be a generated()
+	dl.LocalPath = filepath.Join(dl.CacheDir, dl.ImageName)
+	dl.Sha256sum = info.Sha256Sum
+	dl.URL = downloadURL
+	fcd := FcosDownload{
+		Download: dl,
+	}
+	dataDir, err := GetDataDir(dl.VMKind)
+	if err != nil {
+		return nil, err
+	}
+	fcd.Download.LocalUncompressedFile = fcd.GetLocalUncompressedFile(dataDir)
+	return fcd, nil
+}
+
+// AcquireVMImage determines if the image is already in a FCOS stream. If so,
+// retrieves the image path of the uncompressed file. Otherwise, the user has
+// provided an alternative image, so we set the image path and download the image.
+func (dl Download) AcquireVMImage(imagePath string) (*VMFile, FCOSStream, error) {
+	var (
+		err           error
+		imageLocation *VMFile
+		fcosStream    FCOSStream
+	)
+	switch imagePath {
+	// TODO these need to be re-typed as FCOSStreams
+	case Testing.String(), Next.String(), Stable.String(), "":
+		// Get image as usual
+		fcosStream, err = FCOSStreamFromString(imagePath)
+		if err != nil {
+			return nil, 0, err
+		}
+
+		dd, err := dl.NewFcosDownloader(fcosStream)
+		if err != nil {
+			return nil, 0, err
+		}
+
+		imageLocation, err = NewMachineFile(dd.Get().LocalUncompressedFile, nil)
+		if err != nil {
+			return nil, 0, err
+		}
+
+		if err := DownloadImage(dd); err != nil {
+			return nil, 0, err
+		}
+	default:
+		// The user has provided an alternate image which can be a file path
+		// or URL.
+		fcosStream = CustomStream
+		imgPath, err := dl.AcquireAlternateImage(imagePath)
+		if err != nil {
+			return nil, 0, err
+		}
+		imageLocation = imgPath
+	}
+	return imageLocation, fcosStream, nil
+}
