@@ -418,7 +418,8 @@ var _ = Describe("Podman pull", func() {
 		dirpath := filepath.Join(podmanTest.TempDir, "cirros")
 		err = os.MkdirAll(dirpath, os.ModePerm)
 		Expect(err).ToNot(HaveOccurred())
-		imgPath := fmt.Sprintf("oci:%s", dirpath)
+		imgName := "localhost/name:tag"
+		imgPath := fmt.Sprintf("oci:%s:%s", dirpath, imgName)
 
 		session := podmanTest.Podman([]string{"push", "cirros", imgPath})
 		session.WaitWithDefaultTimeout()
@@ -429,10 +430,9 @@ var _ = Describe("Podman pull", func() {
 		session = podmanTest.Podman([]string{"pull", imgPath})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		session = podmanTest.Podman([]string{"images"})
+		session = podmanTest.Podman([]string{"image", "exists", imgName})
 		session.WaitWithDefaultTimeout()
 		Expect(session).Should(Exit(0))
-		Expect(session.LineInOutputContainsTag(filepath.Join("localhost", dirpath), "latest")).To(BeTrue())
 	})
 
 	It("podman pull + inspect from unqualified-search registry", func() {
@@ -648,11 +648,12 @@ var _ = Describe("Podman pull", func() {
 			podmanTest.AddImageToRWStore(ALPINE)
 
 			bbdir := filepath.Join(podmanTest.TempDir, "busybox-oci")
-			imgPath := fmt.Sprintf("oci:%s", bbdir)
+			imgName := "localhost/name:tag"
+			imgPath := fmt.Sprintf("oci:%s:%s", bbdir, imgName)
 
 			session := decryptionTestHelper(imgPath)
 
-			Expect(session.LineInOutputContainsTag(filepath.Join("localhost", bbdir), "latest")).To(BeTrue())
+			Expect(session.LineInOutputContainsTag("localhost/name", "tag")).To(BeTrue())
 		})
 
 		It("From local registry", func() {
